/*
  TimerTable window for AviCap

  written 2003 by Alexander Rawass (alexannika@users.sourceforge.net)
*/

#include "vidconf.h"
#include "v4lwindow.h"
#include "capproc.h"
#include "codecdialog.h"

#include <avm_fourcc.h>
#include <utils.h>
#include <cpuinfo.h>
#include <creators.h>
#define DECLARE_REGISTRY_SHORTCUT
#include <configfile.h>

#include <qcheckbox.h>
#include <qcombobox.h>
#include <qfiledialog.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qtabwidget.h>
#include <qlistview.h>
#include <qdatetimeedit.h>
#include <qcombobox.h>
#include <qsplitter.h>
#include <qgroupbox.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qmessagebox.h>

#include <stdio.h>
#include <stdlib.h> // atof
#include <qtabwidget.h>

#include <stdio.h>
#include <stdlib.h> // atof

#include "avicapwnd.h"
#include "timertable.h"

void TimerTableItem::setTable(int ch,QDateTime start,QTime stop,QString fname) {
  channel_nr=ch;
  start_time=start;
  stop_time=stop;
  filename=fname;
  
  stop_datetime=timer_window->calc_stopDateTime(start_time,stop_time);

  QString start_timestr=start_time.toString(my_dateformat);
  QString stop_timestr=stop_time.toString(my_timeformat);

  setText(0,QString::number(channel_nr));
  setText(1,my_v4lwin->getXawtvStation(channel_nr).c_str());
  setText(2,start_timestr);
  setText(3,stop_timestr);
  setText(4,filename);

}

TimerTableItem::TimerTableItem(QListView *parent,TimerTable *timer_win,V4LWindow *v4lw,int ch,QDateTime start,QTime stop,QString fname) : QCheckListItem(parent,"test",QCheckListItem::CheckBox){
  timer_window=timer_win;
  my_v4lwin=v4lw;

  setTable(ch,start,stop,fname);
}

TimerTableItem::~TimerTableItem(){
}

TimerTable::TimerTable( QWidget* parent, v4lxif* v4l, V4LWindow* w)
  : QDialog( NULL, "Avicap Timertable", false)
{

  setMinimumSize(600,600);
  setMaximumSize(800,800);
  setCaption("AviCap Timertable");

  timer_mode=TimerOff;
  next_recording=NULL;
  current_recording=NULL;
  last_item_clicked=NULL;
  shutdown_message=NULL;

  my_v4lxif=v4l;
  my_v4lw=w;

  splitter1=new QSplitter(QSplitter::Vertical,this);
  splitter1->setOpaqueResize(true);

  table=new QListView(splitter1);
  table->setBaseSize(600,200);
  table->setMinimumSize(600,200);
  table->setMaximumSize(800,800);
  table->setSelectionMode(QListView::Single);
  connect(table,SIGNAL(pressed(QListViewItem *)),this,SLOT(item_clicked(QListViewItem *)));

  table->addColumn("ChNr.");
  table->addColumn("Channel Name");
  table->addColumn("Start Recording");
  table->addColumn("End Recording");
  table->setColumnAlignment(3,Qt::AlignRight);
  table->addColumn("Filename");

  all_box=new QGroupBox(1,Qt::Horizontal,splitter1);
  all_box->setBaseSize(600,600);
  all_box->setMinimumSize(600,600);
  all_box->setMaximumSize(600,800);

  splitter1->setResizeMode(all_box,QSplitter::Stretch);
  splitter1->setResizeMode(table,QSplitter::Stretch);

  //edit_box=new QGroupBox(2,Qt::Horizontal,all_box);
  edit_box=new QWidget(all_box);
  //edit_box=new QGroupBox(splitter1);
  edit_box->setBaseSize(600,200);


    QGridLayout* gl = new QGridLayout( edit_box, 1, 1 );
    gl->setSpacing( 5 );
    gl->setMargin( 5 );

    int row=0;

    QLabel *l=new QLabel("Start Date/Time:",edit_box);
    gl->addWidget(l,row,0);

  begin_timeedit=new QDateTimeEdit(edit_box,"start_time");
  gl->addWidget(begin_timeedit,row,1);

  row++;
    QLabel *l2=new QLabel("End Time:",edit_box);
    gl->addWidget(l2,row,0);
  end_timeedit=new QTimeEdit(edit_box,"end_time");
  gl->addWidget(end_timeedit,row,1);

  row++;
    QLabel *l3=new QLabel("Channel:",edit_box);
    gl->addWidget(l3,row,0);
  channelbox=new QComboBox(edit_box,"channel");
  gl->addWidget(channelbox,row,1);

    if (my_v4lw->getStationSize())
    {
	for (int i = 0; i < my_v4lw->getStationSize(); i++)
	  channelbox->insertItem(my_v4lw->getXawtvStation(i).c_str());
    }

  row++;
    QLabel *l4=new QLabel("Filename:",edit_box);
    gl->addWidget(l4,row,0);
  filename_edit=new QLineEdit(edit_box,"filename");
  gl->addWidget(filename_edit,row,1);

  edit_box->hide();

    edit_buttons=new QButtonGroup(3,Qt::Horizontal,all_box);

    new_button=new QPushButton("new",edit_buttons);
    modify_button=new QPushButton("modify",edit_buttons);
    remove_button=new QPushButton("remove",edit_buttons);

    rec_buttons=new QButtonGroup(2,Qt::Horizontal,all_box);

    rec_start_button=new QPushButton("start timer",rec_buttons);

    shutdown_checkbox=new QCheckBox("shutdown when finished",rec_buttons);

    connect(new_button,SIGNAL(pressed()),this,SLOT(button_new_pressed()));
    connect(modify_button,SIGNAL(pressed()),this,SLOT(button_modify_pressed()));
    connect(remove_button,SIGNAL(pressed()),this,SLOT(button_remove_pressed()));

    connect(rec_start_button,SIGNAL(pressed()),this,SLOT(button_start_pressed()));

    QWidget *wi=new QWidget(all_box);
    QVBoxLayout *lo=new QVBoxLayout(wi);
    status_label=new QLabel(wi);
    lo->addWidget(status_label);


    load();
    if(table->childCount()>0){
      edit_box->show();

      TimerTableItem *first_item=(TimerTableItem *)table->firstChild();
      table->setCurrentItem(first_item);
      table->setSelected(first_item,true);
      last_item_clicked=first_item;
    }

    init_entry_widgets();
    set_status_label();

    my_timer=new QTimer(this);
    connect(my_timer,SIGNAL(timeout()),SLOT(timer()));
    my_timer->start(1000);
}

TimerTable::~TimerTable(){
  my_timer->stop();
  save();
}

void TimerTable::add_current_time_info(char *bigbuf){
  char sbuf[300];
  
  sprintf(sbuf,"Current Time : %s\n",QDateTime::currentDateTime().toString(my_dateformat).latin1());
  strcat(bigbuf,sbuf);
}

void TimerTable::add_next_recording_info(char *bigbuf){
  char sbuf[500];

  QDateTime currenttime=QDateTime::currentDateTime();

    if(next_recording){
      QDateTime starttime=next_recording->getStartTime();

      int diff_in_secs=currenttime.secsTo(starttime);
      //printf("%d secs to wait\n",diff_in_secs);

      QTime notime=QTime();

      QTime eta=notime.addSecs(diff_in_secs);
      
      int chnr=next_recording->getChannel();
      
      avm::string chname=my_v4lw->getXawtvStation(chnr);
      
      QString fname=next_recording->getFilename();

      sprintf(sbuf,"Next Recording starts at %s in %s\nChannel Nr %d  %s\nFilename %s\n",starttime.toString(my_dateformat).latin1(),eta.toString(my_timeformat).latin1(),chnr,chname.c_str(),fname.latin1());
    }
    else{
      sprintf(sbuf,"No Next Recording\n");
    }
    strcat(bigbuf,sbuf);
}

void TimerTable::add_current_recording_info(char *bigbuf){
  char sbuf[500];
  QDateTime currenttime=QDateTime::currentDateTime();

    if(current_recording){
      QDateTime stoptime=current_recording->getStopDateTime();

      int diff_in_secs=currenttime.secsTo(stoptime);
      //printf("%d secs to go\n",diff_in_secs);

      QTime notime=QTime();

      QTime eta=notime.addSecs(diff_in_secs);
      
      int chnr=current_recording->getChannel();
      
      avm::string chname=my_v4lw->getXawtvStation(chnr);
      
      QString fname=current_recording->getFilename();

      sprintf(sbuf,"Currently Recording: stops at %s in %s\nChannel Nr %d  %s\nFilename %s\n",stoptime.toString(my_dateformat).latin1(),eta.toString(my_timeformat).latin1(),chnr,chname.c_str(),fname.latin1());
    }
    else{
      sprintf(sbuf,"No Current Recording\n");
    }
    strcat(bigbuf,sbuf);
}

void TimerTable::add_sanity_check_info(char *bigbuf){
  char sbuf[400];

  TimerTableItem *item1=(TimerTableItem *)table->firstChild();

  bool insane=false;
  while(!insane && item1!=NULL){
    if(item1->isOn()){
      TimerTableItem *item2=(TimerTableItem *)table->firstChild();

      while(!insane && item2!=NULL){
	if(item2!=item1 && item2->isOn()){
	  QDateTime start1=item1->getStartTime();
	  QDateTime stop1=item1->getStopDateTime();

	  QDateTime start2=item2->getStartTime();
	  QDateTime stop2=item2->getStopDateTime();
	  
	  if((start1<start2 && start2<stop1) || (start2<start1 && start1<stop2)){
	    // do I have to do more sanity checks???
	    avm::string chn1=my_v4lw->getXawtvStation(item1->getChannel());
	    avm::string chn2=my_v4lw->getXawtvStation(item2->getChannel());

	    sprintf(sbuf,"sanity check failed:\na) %10s from %s to %s\nb) %10s from %s to %s\n",chn1.c_str(), \
		   item1->getStartTime().toString(my_dateformat).latin1(), \
		   item1->getStopDateTime().toString(my_dateformat).latin1(),
		   chn2.c_str(),
		   item2->getStartTime().toString(my_dateformat).latin1(), \
		   item2->getStopDateTime().toString(my_dateformat).latin1());
	    strcat(bigbuf,sbuf);
	    insane=true;
	  }
	}
	item2=(TimerTableItem *)item2->nextSibling();
      }
    }

    item1=(TimerTableItem *)item1->nextSibling();
  }
}

void TimerTable::set_status_label(){
    char bigbuf[1000];
    bigbuf[0]='\0';

    QDateTime currenttime=QDateTime::currentDateTime();

    if(timer_mode==TimerOff){
      sprintf(bigbuf,"Current Time: %s\nTimer not activated\n",currenttime.toString(my_dateformat).latin1());
      status_label->setText(bigbuf);
    }
  else if(timer_mode==TimerWaiting){

    add_current_time_info(bigbuf);
    add_next_recording_info(bigbuf);
    add_sanity_check_info(bigbuf);

    status_label->setText(bigbuf);
  }
  else if(timer_mode==TimerRecording){

    add_current_time_info(bigbuf);
    add_current_recording_info(bigbuf);
    add_next_recording_info(bigbuf);
    add_sanity_check_info(bigbuf);

    status_label->setText(bigbuf);
  }
}

void TimerTable::init_entry_widgets(){
  begin_timeedit->setDateTime(QDateTime::currentDateTime());

  end_timeedit->setTime(QTime::currentTime());

  filename_edit->setText("./movie.avi");

  channelbox->setCurrentItem(0);
}

void TimerTable::item_clicked(QListViewItem *lvitem){
  TimerTableItem *item=(TimerTableItem *)lvitem;

  if(item!=NULL){
    begin_timeedit->setDateTime(item->getStartTime());

    end_timeedit->setTime(item->getStopTime());

    filename_edit->setText(item->getFilename());

    channelbox->setCurrentItem(item->getChannel());

    last_item_clicked=item;
  }
}

void TimerTable::button_new_pressed(){
  edit_box->show();
  init_entry_widgets();
  TimerTableItem *newitem=set_tableitem(NULL);

  table->setCurrentItem(newitem);
  table->setSelected(newitem,true);
  last_item_clicked=newitem;
}

TimerTableItem * TimerTable::set_tableitem(TimerTableItem *item){
  QDateTime start_time=begin_timeedit->dateTime();
  QTime stop_time=end_timeedit->time();

  int channel_nr=channelbox->currentItem();

  QString filename=filename_edit->text();

  QString stationname=my_v4lw->getXawtvStation(channel_nr).c_str();

  QString start_timestr=start_time.toString(my_dateformat);
  QString stop_timestr=stop_time.toString(my_timeformat);

  if(item!=NULL){
    item->setTable(channel_nr,start_time,stop_time,filename);
  }
  else{
    item=new TimerTableItem(table,this,my_v4lw,channel_nr,start_time,stop_time,filename);
  }

    return item;
}

void TimerTable::button_modify_pressed(){
  //    TimerTableItem *item=(TimerTableItem *)table->currentItem();

  TimerTableItem *item=last_item_clicked;

    if(item!=NULL){
      set_tableitem(item);
    }

    save();
}


void TimerTable::button_remove_pressed(){
  TimerTableItem *item=(TimerTableItem *)table->currentItem();

  if(item==current_recording){
    printf("can't remove - it's the current recording\n");
  }
  else{
    if(item==next_recording){
      next_recording=NULL;
    }
    if(item==last_item_clicked){
      last_item_clicked=NULL;
    }
    table->takeItem(item);
    delete item;
    save();
  }
}

TimerTableItem  *TimerTable::find_first_recording(){
  TimerTableItem *item=(TimerTableItem *)table->firstChild();

  QDateTime currenttime=QDateTime::currentDateTime();
  QDateTime found_date=currenttime;

  TimerTableItem *found_item=NULL;

  while(item!=NULL){
    if(item->isOn()){
      QDateTime new_date=item->getStartTime();

      if(new_date>currenttime){
	// we ignore recordings in the past
	if(new_date<found_date || found_date==currenttime){
	  found_date=new_date;
	  found_item=item;
	}
      }
    }
    
    item=(TimerTableItem *)item->nextSibling();
  }

  if(found_item){
    if(found_date>currenttime){
      return found_item;
    }

    return NULL;
  }

  return NULL;
}

void TimerTable::button_start_pressed(){
  save();

  if(timer_mode==TimerOff){
    if(table->childCount()>0){

      TimerTableItem *first_item=find_first_recording();

      if(first_item!=NULL){
	next_recording=first_item;

	timer_mode=TimerWaiting;
	rec_start_button->setText("Stop Timer");
      }
    }
    else{
      printf("no programs, timer not started\n");
    }
  }
  else if(timer_mode==TimerRecording){
    printf("can't switch off timer while recording with timer\n");
  }
  else if(timer_mode==TimerWaiting){
    timer_mode=TimerOff;
    rec_start_button->setText("Start Timer");
    next_recording=NULL;
  }
  set_status_label();
}

void TimerTable::timer(){

  next_recording=find_first_recording();

  if(next_recording && timer_mode==TimerWaiting){
    QDateTime  start_time=next_recording->getStartTime();

    QDateTime currenttime=QDateTime::currentDateTime();

    int secs_to_go=currenttime.secsTo(start_time);

    if(secs_to_go<10){
      // we start recording actually 10 secs before the time
   
      // stop old recording if necessary
      stop_current_recording();


   my_v4lw->setXawtv(next_recording->getChannel());

   CaptureConfig *conf=new CaptureConfig();
   conf->load();
   conf->setFilename(next_recording->getFilename().latin1());

   my_v4lw->captureAVI(conf); //make the new capture window

   AviCapDialog *dialog=my_v4lw->getCaptureDialog();

   if(dialog!=NULL){
     dialog->start(); // run the thing
     timer_mode=TimerRecording;
     current_recording=next_recording;
   }
 }
  }
  else if(timer_mode==TimerRecording && current_recording){
    //check if we have to switch off current recording
    QDateTime  stop_time=current_recording->getStopDateTime();

    QDateTime currenttime=QDateTime::currentDateTime();

    //int secs_to_go=currenttime.secsTo(stop_time);

    //printf("%d secs to go to stop recording\n",secs_to_go);
    if(currenttime  > stop_time){
      stop_current_recording();
      current_recording=NULL;
      timer_mode=TimerWaiting;
    }

  }
  else if(timer_mode==TimerWaiting && next_recording==NULL && current_recording==NULL){
    // no more recordings, can we shutdown?

    bool shutdown=shutdown_checkbox->isChecked();
    
    if(shutdown && !shutdown_message){
      // open up a new message
      printf("shutdown!\n");
      shutdown_message=new QMessageBox("AviCap Shutdown warning","System is going to shutdown\n\nin some seconds\n",QMessageBox::Critical,QMessageBox::Abort,QMessageBox::NoButton,QMessageBox::NoButton,this);
      shutdown_message->show();
      shutdown_time=QDateTime::currentDateTime();
      shutdown_time=shutdown_time.addSecs(3*60);
    }
    else if(shutdown && shutdown_message){
      QDateTime currenttime=QDateTime::currentDateTime();
      int secsto=currenttime.secsTo(shutdown_time);
      printf("secs to shutdown: %d res=%d\n",secsto,shutdown_message->result());
      if(shutdown_message->result()==5){
	// abort shutdown
	delete shutdown_message;
	shutdown_message=NULL;
	shutdown_checkbox->setChecked(0);
      }
      else if(secsto>0){
	char buf[300];
	sprintf(buf,"System is going to shutdown\n\n in %d seconds\n",secsto);
	shutdown_message->setText(buf);
	shutdown_message->show();
      }
      else{
	save();

	// SHUTDOWN NOW

	system("sudo avicap-shutdown &");

	//exit avicap
	exit(0);
      }
    }
  }
 set_status_label();
}

void TimerTable::stop_current_recording(){
     // stop old recording
   AviCapDialog *dialog_old=my_v4lw->getCaptureDialog();
   if(dialog_old!=NULL){
     CaptureProcess *capproc=dialog_old->getCaptureProcess();

     if(capproc!=NULL){
       // there's a recording running
       dialog_old->stop();
       dialog_old->close();
     }
   }
}

QDateTime TimerTable::calc_stopDateTime(QDateTime start_datetime,QTime stop_time){
  QTime start_time=start_datetime.time();
  QDate start_date=start_datetime.date();

  QDateTime stop_datetime(start_date,stop_time);

  if(start_time > stop_time ){
    // the recording ends the next day
    stop_datetime=stop_datetime.addDays(1);
  }

  return stop_datetime;
}

void TimerTable::load(){
    int i=0;
    char buf[300];

    sprintf(buf,"TT-%02d-StartTime",i);
    avm::string starttimestr=RS(buf,"NONE");

    while(starttimestr!="NONE"){
      QDateTime starttime=QDateTime::fromString(starttimestr.c_str());

      sprintf(buf,"TT-%02d-StopTime",i);
      avm::string stoptimestr=RS(buf,"NONE");
      QTime stoptime=QTime::fromString(stoptimestr.c_str());

      sprintf(buf,"TT-%02d-Channel",i);
      int channel=RI(buf,0);

      sprintf(buf,"TT-%02d-Filename",i);
      avm::string filename=RS(buf,"./movie.avi");
      
      TimerTableItem *item=new TimerTableItem(table,this,my_v4lw,channel,starttime,stoptime,filename.c_str());

      sprintf(buf,"TT-%02d-Active",i);
      int ison=RI(buf,1);

      item->setOn(ison);
      
      i++;

      sprintf(buf,"TT-StartTime-%02d",i);
      starttimestr=RS(buf,"NONE");

    } 
 

}

void TimerTable::save(){
  int i=0;
  char buf[200];

  TimerTableItem *item=(TimerTableItem *)table->firstChild();

  while(item!=NULL){
    sprintf(buf,"TT-%02d-StartTime",i);
    //WS(buf,item->getStartTime().toString(my_dateformat).latin1());
    WS(buf,item->getStartTime().toString().latin1());
    sprintf(buf,"TT-%02d-StopTime",i);
    WS(buf,item->getStopTime().toString().latin1());
    sprintf(buf,"TT-%02d-Channel",i);
    WI(buf,item->getChannel());
    sprintf(buf,"TT-%02d-Filename",i);
    WS(buf,item->getFilename().latin1());
    sprintf(buf,"TT-%02d-Active",i);
    WI(buf,item->isOn());
    
    item=(TimerTableItem *)item->nextSibling();
    i++;
  }

  // make sure that's the last one...

    sprintf(buf,"TT-StartTime-%02d",i);
    WS(buf,"NONE");

}
		       
#include "timertable.moc"
