/*
  TimerTable window for AviCap

  written 2003 by Alexander Rawass (alexannika@users.sourceforge.net)
*/

#ifndef timertable_h
#define timertable_h

#include <qwidget.h>
#include <qtimer.h>
#include <qlistview.h>

#include <avm_stl.h>
#include <avm_locker.h>

class v4lxif;
class V4LWindow;

class QPopupMenu;
class QLabel;
class QListView;
class QComboBox;
class QDateTimeEdit;
class QTimeEdit;
class QSplitter;
class QGroupBox;
class QPushButton;
class QButtonGroup;
class QLineEdit;
class QListViewItem;
class QFrame;
class QMessageBox;

class TimerTable;

static QString my_timeformat="hh:mm:ss";
static QString my_dateformat=("dd MMM yyyy    "+my_timeformat);

enum TimerMode { TimerOff,TimerWaiting,TimerRecording };

//#define my_tabletype QListView
//#define my_tableitemtype QListViewItem

class TimerTableItem: public QCheckListItem
{

public:
  TimerTableItem(QListView *parent,TimerTable *timer_win,V4LWindow *v4lw,int ch,QDateTime start,QTime stop,QString fname);
  //TimerTableItem(QWidget *parent);
  ~TimerTableItem();

  void  setTable(int ch,QDateTime start,QTime stop,QString fname);

 private:
  QDateTime start_time;
  QTime  stop_time;
  QDateTime stop_datetime;
  int channel_nr;
  QString filename;
  TimerTable *timer_window;
  V4LWindow *my_v4lwin;

 public:
  QDateTime getStartTime() { return start_time; };
  QTime getStopTime() { return stop_time; };
  QDateTime getStopDateTime() { return stop_datetime; };
  QString getFilename() { return filename; };
  int getChannel() { return channel_nr; };


};

class TimerTable: public QDialog
{
    Q_OBJECT
public:

    TimerTable( QWidget* parent, v4lxif* v4l, V4LWindow* w);
    ~TimerTable();
  void save();
  void load();
  QDateTime calc_stopDateTime(QDateTime start_datetime,QTime stop_time);

public slots:
  void button_new_pressed();
  void button_remove_pressed();
  void button_modify_pressed();
  void button_start_pressed();
  void item_clicked(QListViewItem *item);
  void timer();

protected:

private:
  void   init_entry_widgets();
  void   set_status_label();
  TimerTableItem *find_first_recording();
  TimerTableItem *   set_tableitem(TimerTableItem *item);
  void stop_current_recording();
  void  add_current_time_info(char *bigbuf);
  void  add_current_recording_info(char *bigbuf);
  void  add_next_recording_info(char *bigbuf);
  void  add_sanity_check_info(char *bigbuf);

  TimerMode timer_mode;

 v4lxif *my_v4lxif;
 V4LWindow *my_v4lw;

 QSplitter *splitter1;
 //QGroupBox  *edit_box;
 QWidget  *edit_box;
 QGroupBox  *all_box;

 QListView *table;
 QComboBox *channelbox;

 QDateTimeEdit *begin_timeedit;
 QTimeEdit *end_timeedit;
 QLineEdit *filename_edit;

 QButtonGroup *edit_buttons;
 QPushButton *new_button;
 QPushButton *modify_button;
 QPushButton *remove_button;

 QButtonGroup *rec_buttons;
 QPushButton *rec_start_button;
 QPushButton *rec_stop_button;
 QCheckBox *shutdown_checkbox;

 QFrame *status_frame;
 QLabel *status_label;

 QMessageBox *shutdown_message;
 QDateTime shutdown_time;

 TimerTableItem *next_recording;
 TimerTableItem *current_recording;
 TimerTableItem *last_item_clicked;

 QTimer *my_timer;
};

#endif
